import os
from unittest import mock
import tempfile

import pytest
from requests import exceptions as req_exp
from requests.models import Response
from unittest.mock import patch
from uk_bin_collection.get_bin_data import AbstractGetBinDataClass as agbdc
from uk_bin_collection.get_bin_data import setup_logging
import logging


def mocked_requests_get(*args, **kwargs):
    class MockResponse:
        def __init__(self, json_data, status_code, raise_error_type):
            self.text = json_data
            self.status_code = status_code
            if raise_error_type is not None:
                self.raise_for_status = self.raise_error(raise_error_type)
            else:
                self.raise_for_status = lambda: None

        def raise_error(self, errorType):
            if errorType == "HTTPError":
                raise req_exp.HTTPError()
            elif errorType == "ConnectionError":
                raise req_exp.ConnectionError()
            elif errorType == "Timeout":
                raise req_exp.Timeout()
            elif errorType == "RequestException":
                raise req_exp.RequestException()
            return errorType

    if args[0] == "aurl":
        return MockResponse({"test_data": "test"}, 200, None)
    elif args[0] == "HTTPError":
        return MockResponse({}, 999, "HTTPError")
    elif args[0] == "ConnectionError":
        return MockResponse({}, 999, "ConnectionError")
    elif args[0] == "Timeout":
        return MockResponse({}, 999, "Timeout")
    elif args[0] == "RequestException":
        return MockResponse({}, 999, "RequestException")
    elif args[0] == "notPage":
        return MockResponse("not json", 200, None)
    return MockResponse(None, 404, "HTTPError")


# Unit tests


def test_logging_exception():
    logging_dict = "SW1A 1AA"
    with pytest.raises(ValueError) as exc_info:
        result = setup_logging(logging_dict, "ROOT")
    assert exc_info.typename == "ValueError"


def test_setup_logging_valid_config():
    # Example of a minimal valid logging configuration dictionary
    logging_config = {
        "version": 1,
        "handlers": {
            "console": {
                "class": "logging.StreamHandler",
                "level": "DEBUG",
            },
        },
        "loggers": {
            "ROOT": {
                "handlers": ["console"],
                "level": "DEBUG",
            },
        },
    }
    logger_name = "ROOT"
    # Run the function with valid logging configuration
    logger = setup_logging(logging_config, logger_name)

    # Assert that logger is correctly configured
    assert logger.name == logger_name
    assert logger.level == logging.DEBUG


@mock.patch("requests.get", side_effect=mocked_requests_get)
def test_get_data(mock_get):
    page_data = agbdc.get_data("aurl")
    assert page_data.text == {"test_data": "test"}


@pytest.mark.parametrize(
    "url", ["HTTPError", "ConnectionError", "Timeout", "RequestException"]
)
@mock.patch("requests.get", side_effect=mocked_requests_get)
def test_get_data_error(mock_get, url):
    with pytest.raises(Exception) as exc_info:
        result = agbdc.get_data(url)
    assert exc_info.typename == url


def test_output_json():
    bin_data = {"bin": ""}
    output = agbdc.output_json(bin_data)
    assert type(output) == str
    assert output == '{\n    "bin": ""\n}'


class ConcreteGetBinDataClass(agbdc):
    """Concrete implementation of the abstract class to test abstract methods."""

    def parse_data(self, page: str, **kwargs) -> dict:
        return {"mock_key": "mock_value"}

    def update_dev_mode_data(self, council_module_str, this_url, **kwargs):
        # You can implement the method or delegate it to the abstract class's method
        super().update_dev_mode_data(council_module_str, this_url, **kwargs)


@pytest.fixture
def concrete_class_instance():
    return ConcreteGetBinDataClass()


def test_get_and_parse_data_no_skip_get_url(concrete_class_instance):
    mock_page = "mocked page content"
    mock_parsed_data = {"mock_key": "mock_value"}

    with mock.patch.object(
        concrete_class_instance, "get_data", return_value=mock_page
    ) as mock_get_data, mock.patch.object(
        concrete_class_instance, "parse_data", return_value=mock_parsed_data
    ) as mock_parse_data:

        result = concrete_class_instance.get_and_parse_data("http://example.com")

        mock_get_data.assert_called_once_with("http://example.com")
        mock_parse_data.assert_called_once_with(mock_page, url="http://example.com")
        assert result == mock_parsed_data


def test_get_and_parse_data_skip_get_url(concrete_class_instance):
    mock_parsed_data = {"mock_key": "mock_value"}

    with mock.patch.object(
        concrete_class_instance, "parse_data", return_value=mock_parsed_data
    ) as mock_parse_data:

        result = concrete_class_instance.get_and_parse_data(
            "http://example.com", skip_get_url=True
        )

        mock_parse_data.assert_called_once_with(
            "", url="http://example.com", skip_get_url=True
        )
        assert result == mock_parsed_data


@pytest.fixture
def setup_test_update_dev_mode_data():
    """Fixture to set up and tear down the environment for test_update_dev_mode_data"""
    # Create a temporary directory to simulate the working directory
    test_dir = tempfile.TemporaryDirectory()

    # Patch os.getcwd() to return the temporary directory
    cwd_patch = patch("os.getcwd", return_value=test_dir.name)
    mock_getcwd = cwd_patch.start()

    # Ensure the nested directory structure exists
    os.makedirs(os.path.join(test_dir.name, "uk_bin_collection", "tests"))

    # Yield control back to the test, then clean up after the test
    yield test_dir.name  # Provide the test with the temporary directory

    # Teardown
    test_dir.cleanup()
    cwd_patch.stop()


def test_update_dev_mode_data(setup_test_update_dev_mode_data):
    """Test update_dev_mode_data method to ensure input.json is updated correctly"""
    # The setup fixture returns the mocked current working directory
    mock_cwd = setup_test_update_dev_mode_data

    # Create an instance of the concrete class that inherits from AbstractGetBinDataClass
    obj = ConcreteGetBinDataClass()

    # Define input arguments for the method
    council_module_str = "test_council_module"
    this_url = "https://example.com"
    kwargs = {
        "postcode": "12345",
        "paon": "1A",
        "uprn": "100012345",
        "usrn": "200012345",
        "web_driver": "mocked_web_driver",
        "skip_get_url": True,
    }

    # Call the method being tested on the instance
    obj.update_dev_mode_data(council_module_str, this_url, **kwargs)

    # Verify that input.json was created in the correct location
    input_file_path = os.path.join(mock_cwd, "uk_bin_collection", "tests", "input.json")
    assert os.path.exists(input_file_path)

    # Read the contents of the file and make necessary assertions
    with open(input_file_path, "r") as f:
        file_content = f.read()

    # Example assertion - check if certain values exist in the file content (based on your actual file format)
    assert "100012345" in file_content  # Checking UPRN as an example



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = "https://harborough.fccenvironment.co.uk/detail-address"

        headers = {
            "Content-Type": "application/json",
            "User-Agent": "Mozilla/5.0",
            "Referer": "https://harborough.fccenvironment.co.uk/",
        }
        params = {"Uprn": user_uprn}
        response = requests.post(URI, headers=headers, json=params)

        soup = BeautifulSoup(response.content, features="html.parser")
        bin_collection = soup.find(
            "div", {"class": "blocks block-your-next-scheduled-bin-collection-days"}
        )
        lis = bin_collection.find_all("li")
        for li in lis:
            try:
                split = re.match(r"(.+)\s(\d{1,2} \w+ \d{4})$", li.text)
                bin_type = split.group(1).strip()
                date = split.group(2)

                dict_data = {
                    "type": bin_type,
                    "collectionDate": datetime.strptime(
                        date,
                        "%d %B %Y",
                    ).strftime("%d/%m/%Y"),
                }
                bindata["bins"].append(dict_data)
            except:
                continue

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



from time import sleep

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select, WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the base
    class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_uprn(user_uprn)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(
                "https://my.northdevon.gov.uk/service/WasteRecyclingCollectionCalendar"
            )

            # Wait for iframe to load and switch to it
            WebDriverWait(driver, 30).until(
                EC.frame_to_be_available_and_switch_to_it((By.ID, "fillform-frame-1"))
            )

            # Wait for postcode entry box
            postcode = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.ID, "postcode_search"))
            )
            # Enter postcode
            postcode.send_keys(user_postcode.replace(" ", ""))

            # Wait for address selection dropdown to appear
            address = Select(
                WebDriverWait(driver, 10).until(
                    EC.visibility_of_element_located((By.ID, "chooseAddress"))
                )
            )

            # Wait for spinner to disappear (signifies options are loaded for select)
            WebDriverWait(driver, 10).until(
                EC.invisibility_of_element_located(
                    (By.CLASS_NAME, "spinner-outer")
                )  # row-fluid spinner-outer
            )

            # Sometimes the options aren't fully there despite the spinner being gone, wait another 2 seconds.
            sleep(2)

            # Select address by UPRN
            address.select_by_value(user_uprn)

            # Wait for spinner to disappear (signifies data is loaded)
            WebDriverWait(driver, 10).until(
                EC.invisibility_of_element_located((By.CLASS_NAME, "spinner-outer"))
            )

            sleep(2)

            address_confirmation = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.XPATH, "//h2[contains(text(), 'Your address')]")
                )
            )

            next_button = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.XPATH, "//button/span[contains(@class, 'nextText')]")
                )
            )

            next_button.click()

            results = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.XPATH, "//h4[contains(text(), 'Key')]")
                )
            )

            # Find data table
            data_table = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        '//div[@data-field-name="html1"]/div[contains(@class, "fieldContent")]',
                    )
                )
            )

            # Make a BS4 object
            soup = BeautifulSoup(
                data_table.get_attribute("innerHTML"), features="html.parser"
            )

            # Initialize the data dictionary
            data = {"bins": []}

            # Loop through each list of waste dates
            waste_sections = soup.find_all("ul", class_="wasteDates")

            current_month_year = None

            for section in waste_sections:
                for li in section.find_all("li", recursive=False):
                    if "MonthLabel" in li.get("class", []):
                        # Extract month and year (e.g., "April 2025")
                        header = li.find("h4")
                        if header:
                            current_month_year = header.text.strip()
                    elif any(
                        bin_class in li.get("class", [])
                        for bin_class in ["BlackBin", "GreenBin", "Recycling"]
                    ):
                        bin_type = li.find("span", class_="wasteType").text.strip()
                        day = li.find("span", class_="wasteDay").text.strip()
                        weekday = li.find("span", class_="wasteName").text.strip()

                        if current_month_year and day:
                            try:
                                full_date = f"{day} {current_month_year}"
                                collection_date = datetime.strptime(
                                    full_date, "%d %B %Y"
                                ).strftime(date_format)
                                dict_data = {
                                    "type": bin_type,
                                    "collectionDate": collection_date,
                                }
                                data["bins"].append(dict_data)
                            except Exception as e:
                                print(f"Skipping invalid date '{full_date}': {e}")

            data["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
            )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_postcode = kwargs.get("postcode")
        user_paon = kwargs.get("paon")
        check_postcode(user_postcode)
        check_paon(user_paon)
        bindata = {"bins": []}

        URI = "http://collectiondates.eastlothian.gov.uk/ajax/your-calendar/load-streets-spring-2024.asp"

        payload = {
            "postcode": user_postcode,
        }

        headers = {
            "Referer": "http://collectiondates.eastlothian.gov.uk/your-calendar",
            "User-Agent": "Mozilla/5.0",
        }

        # Make the GET request
        response = requests.get(URI, headers=headers, params=payload)

        # Parse the HTML with BeautifulSoup
        soup = BeautifulSoup(response.text, "html.parser")

        # Find the select dropdown
        select = soup.find("select", id="SelectStreet")

        # Find the option that contains "Flat 1"
        address = select.find("option", string=lambda text: text and user_paon in text)

        URI = "http://collectiondates.eastlothian.gov.uk/ajax/your-calendar/load-recycling-summer-2024.asp"

        payload = {
            "id": address["value"],
        }

        # Make the GET request
        response = requests.get(URI, headers=headers, params=payload)

        # Parse the HTML with BeautifulSoup
        soup = BeautifulSoup(response.text, "html.parser")

        # Extract collection details
        calendar_items = soup.find_all("div", class_="calendar-item")
        for item in calendar_items:
            waste_label = item.find("div", class_="waste-label").text.strip()
            waste_value = item.find("div", class_="waste-value").find("h4").text.strip()

            try:
                collection_date = datetime.strptime(
                    remove_ordinal_indicator_from_date_string(waste_value),
                    "%A %d %B %Y",
                )
            except ValueError:
                continue

            dict_data = {
                "type": waste_label.replace(" is:", ""),
                "collectionDate": collection_date.strftime(date_format),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return bindata



from datetime import datetime

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get and check UPRN
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        API_URL = (
            "https://collections-ardsandnorthdown.azurewebsites.net/WSCollExternal.asmx"
        )

        # council seems to always be ARD no matter what the old council was
        PAYLOAD = f"""<?xml version="1.0" encoding="utf-8" ?>
        <soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xmlns:xsd="http://www.w3.org/2001/XMLSchema"
            xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
            <soap:Body>
                <getRoundCalendarForUPRN  xmlns="http://webaspx-collections.azurewebsites.net/">
                    <council>ARD</council>
                    <UPRN>{user_uprn}</UPRN>
                    <from>Chtml</from>
                </getRoundCalendarForUPRN >
            </soap:Body>
        </soap:Envelope>
        """

        r = requests.post(
            API_URL,
            data=PAYLOAD,
            headers={"Content-Type": "text/xml; charset=utf-8"},
        )
        r.raise_for_status()

        # html unescape text
        text = (
            (r.text.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&"))
            .split("<getRoundCalendarForUPRNResult>")[-1]
            .split("</getRoundCalendarForUPRNResult>")[0]
        )

        soup = BeautifulSoup(text, "html.parser")

        # Initialize dictionary to store bin dates
        bin_schedule = {}

        # Define regex pattern to capture day and date (e.g., Tue 5 Nov)
        date_pattern = re.compile(r"\b\w{3} \d{1,2} \w{3}\b")

        current_year = datetime.now().year

        # Find each bin collection line, parse date, and add to dictionary
        for bin_info in soup.find_all("b"):
            bin_type = bin_info.text.strip()
            bin_details = bin_info.next_sibling.strip() if bin_info.next_sibling else ""
            # Check for "Today" or "Tomorrow"
            if "Today" in bin_details:
                collection_date = datetime.now().strftime("%a %d %b")
                bin_schedule[bin_type] = collection_date
            elif "Tomorrow" in bin_details:
                collection_date = (datetime.now() + timedelta(days=1)).strftime(
                    "%a %d %b"
                )
                bin_schedule[bin_type] = collection_date
            else:
                # Extract date if it's a full date format
                date_match = date_pattern.search(bin_details)
                if date_match:
                    bin_schedule[bin_type] = date_match.group()

        # Display the parsed schedule with dates only
        for bin_type, collection_date in bin_schedule.items():
            date = datetime.strptime(collection_date, "%a %d %b")

            if date.month == 1 and datetime.now().month > 1:
                date = date.replace(year=current_year + 1)
            else:
                date = date.replace(year=current_year)

            dict_data = {
                "type": bin_type,
                "collectionDate": date.strftime("%d/%m/%Y"),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )
        return bindata



from time import sleep

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        user_postcode = kwargs.get("postcode")
        user_paon = kwargs.get("paon")
        check_postcode(user_postcode)
        check_paon(user_paon)

        URI = "https://recyclingservices.brent.gov.uk/waste"

        payload = {"postcode": user_postcode}

        s = requests.Session()

        # Make the POST request
        response = s.post(URI, data=payload)

        # Make a BS4 object
        soup = BeautifulSoup(response.content, features="html.parser")

        address_list = soup.find_all("option")

        current_year = datetime.now().year
        next_year = current_year + 1

        for address in address_list:
            if user_paon in (address.text):
                address_id = address.get("value")
                URI = f"https://recyclingservices.brent.gov.uk/waste/{address_id}"

                counter = 0
                r = s.get(URI)
                while "Loading your bin days..." in r.text:
                    counter = counter + 1
                    if counter == 20:
                        return data
                    sleep(2)
                    r = s.get(URI)

                r.raise_for_status()

                soup = BeautifulSoup(r.content, features="html.parser")

                wastecollections = soup.find("div", {"class": "waste__collections"})

                # Find all waste service sections
                waste_services = wastecollections.find_all(
                    "h3", class_="govuk-heading-m waste-service-name"
                )

                for service in waste_services:
                    # Get the collection type (e.g., Rubbish, Recycling)
                    collection_type = (service.get_text(strip=True)).split("\n")[0]

                    # Find the sibling container holding details
                    service_details = service.find_next(
                        "dl", class_="govuk-summary-list"
                    )

                    if service_details:

                        # Extract next collection date
                        next_collection_row = service_details.find(
                            "dt", string="Next collection"
                        )
                        next_collection = (
                            next_collection_row.find_next_sibling("dd").get_text(
                                strip=True
                            )
                            if next_collection_row
                            else "Unknown"
                        )

                        # Parse dates into standard dd/mm/yyyy format
                        next_collection_date = datetime.strptime(
                            remove_ordinal_indicator_from_date_string(next_collection),
                            "%A, %d %B",
                        )

                        if (datetime.now().month == 12) and (
                            next_collection.month == 1
                        ):
                            next_collection_date = next_collection_date.replace(
                                year=next_year
                            )
                        else:
                            next_collection_date = next_collection_date.replace(
                                year=current_year
                            )

                        dict_data = {
                            "type": collection_type.strip(),
                            "collectionDate": next_collection_date.strftime(
                                date_format
                            ),
                        }
                        data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        api_url = "https://www.hounslow.gov.uk/homepage/86/recycling_and_waste_collection_day_finder"
        user_uprn = kwargs.get("uprn")

        # Check the UPRN is valid
        check_uprn(user_uprn)

        # Create the form data
        form_data = {
            "UPRN": user_uprn,
        }

        # Make a request to the API
        requests.packages.urllib3.disable_warnings()
        response = requests.post(api_url, data=form_data)

        # Make a BS4 object
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        # Get the div element
        div_element = soup.find("div", {"class": "bin_day_main_wrapper"})

        # Get all bins with their corresponding dates using list comprehension
        # This creates a list of tuples, where each tuple contains the bin type and collection date
        bins_with_dates = [
            (
                bin.get_text().strip(),
                h4.get_text().replace("This ", "").replace("Next ", ""),
            )
            # This first for loop iterates over each h4 element
            for h4 in div_element.find_all("h4")
            # This nested for loop iterates over each li element within the corresponding ul element
            for bin in h4.find_next_sibling("ul").find_all("li")
        ]

        for bin_type, collection_date in bins_with_dates:
            if "-" in collection_date:
                date_part = collection_date.split(" - ")[1]
                data["bins"].append(
                    {
                        "type": bin_type,
                        "collectionDate": datetime.strptime(
                            date_part, "%d %b %Y"
                        ).strftime(date_format),
                    }
                )
            elif len(collection_date.split(" ")) == 4:
                data["bins"].append(
                    {
                        "type": bin_type,
                        "collectionDate": datetime.strptime(
                            collection_date, "%A %d %b %Y"
                        ).strftime(date_format),
                    }
                )
            else:
                data["bins"].append(
                    {
                        "type": bin_type,
                        "collectionDate": datetime.strptime(
                            collection_date, "%d %b %Y"
                        ).strftime(date_format),
                    }
                )

        return data



import json
import requests
import sys
import base64
from tabulate import tabulate


def get_council_files(repo, branch):
    """
    Get a list of all .py council files in the 'councils' directory
    from the GitHub repo (via API), plus a mapping from council name
    to the file's GitHub 'download_url' or 'contents_url'.
    """
    url = f"https://api.github.com/repos/{repo}/contents/uk_bin_collection/uk_bin_collection/councils?ref={branch}"
    print(f"Fetching council files from: {url}")
    response = requests.get(url, headers={"Accept": "application/vnd.github.v3+json"})
    if response.status_code == 200:
        data = response.json()
        # data should be a list of items in that folder
        if isinstance(data, list):
            councils = {}
            for item in data:
                name = item["name"]
                if name.endswith(".py"):
                    council_name = name.replace(".py", "")
                    councils[council_name] = item["url"]  # 'url' gives API-based content URL
            return councils
        else:
            raise ValueError("Expected a list from the GitHub response but got something else.")
    else:
        print(f"Failed to fetch councils from files: {response.content}")
        return {}


def get_council_file_content(api_url):
    """
    Given the API URL for a file in GitHub, fetch its content (decoded).
    The 'download_url' is direct raw, but the 'url' is the API URL for the content.
    We'll use the latter, decode base64, and return the text.
    """
    # Example: https://api.github.com/repos/robbrad/UKBinCollectionData/contents/...
    response = requests.get(api_url, headers={"Accept": "application/vnd.github.v3+json"})
    if response.status_code == 200:
        file_json = response.json()
        # file_json["content"] is base64-encoded
        content = file_json.get("content", "")
        decoded = base64.b64decode(content).decode("utf-8")
        return decoded
    else:
        print(f"Failed to fetch file content: {response.content}")
        return ""


def get_input_json_data(repo, branch):
    """
    Fetch the entire input.json from GitHub and return it as a Python dict.
    """
    url = f"https://api.github.com/repos/{repo}/contents/uk_bin_collection/tests/input.json?ref={branch}"
    print(f"Fetching input JSON from: {url}")
    response = requests.get(url, headers={"Accept": "application/vnd.github.v3+json"})
    if response.status_code == 200:
        try:
            file_json = response.json()
            content = file_json.get("content", "")
            decoded = base64.b64decode(content).decode("utf-8")
            data = json.loads(decoded)
            return data
        except json.JSONDecodeError as e:
            print(f"JSON decoding error: {e}")
            raise
    else:
        print(f"Failed to fetch input JSON: {response.content}")
        return {}


def council_needs_update(council_name, json_data, council_file_content):
    """
    Check if the given council needs an update:
      - We say 'needs update' if 'web_driver' is missing in the JSON,
        BUT the script uses 'create_webdriver' in code.
    """
    # If the council isn't in the JSON at all, we can't do the check
    # (or we assume no JSON data => no web_driver?).
    council_data = json_data.get(council_name, {})
    web_driver_missing = ("web_driver" not in council_data)
    create_webdriver_present = ("create_webdriver" in council_file_content)

    return web_driver_missing and create_webdriver_present


def compare_councils(file_council_dict, json_data):
    """
    Compare councils in files vs councils in JSON, check for needs_update,
    and gather everything for final tabulation.

    Returns:
      - all_councils_data: dict keyed by council name:
            {
              "in_files": bool,
              "in_json": bool,
              "discrepancies_count": int,
              "needs_update": bool
            }
      - any_discrepancies_found: bool (if any differences in in_files vs in_json)
      - any_updates_needed: bool (if any council needs update)
    """
    file_councils = set(file_council_dict.keys())
    json_councils = set(json_data.keys())

    all_councils = file_councils.union(json_councils)
    all_council_data = {}

    any_discrepancies_found = False
    any_updates_needed = False

    for council in all_councils:
        in_files = council in file_councils
        in_json = council in json_councils
        # Count how many are False
        discrepancies_count = [in_files, in_json].count(False)

        # If the file is in the repo, fetch its content for checking
        content = ""
        if in_files:
            file_api_url = file_council_dict[council]
            content = get_council_file_content(file_api_url)

        # Evaluate "needs_update" only if the file is in place
        # (If there's no file, you might consider it "False" by default)
        needs_update = False
        if in_files:
            needs_update = council_needs_update(council, json_data, content)

        if discrepancies_count > 0:
            any_discrepancies_found = True
        if needs_update:
            any_updates_needed = True

        all_council_data[council] = {
            "in_files": in_files,
            "in_json": in_json,
            "discrepancies_count": discrepancies_count,
            "needs_update": needs_update,
        }

    return all_council_data, any_discrepancies_found, any_updates_needed


def main(repo="robbrad/UKBinCollectionData", branch="master"):
    print(f"Starting comparison for repo: {repo}, branch: {branch}")

    # 1) Get council file data (dict: { council_name: content_api_url, ... })
    file_council_dict = get_council_files(repo, branch)

    # 2) Get the entire JSON data
    json_data = get_input_json_data(repo, branch)

    # 3) Compare
    (
        all_councils_data,
        discrepancies_found,
        updates_needed,
    ) = compare_councils(file_council_dict, json_data)

    # 4) Print results
    table_data = []
    headers = ["Council Name", "In Files", "In JSON", "Needs Update?", "Discrepancies"]
    # Sort councils so that ones with the highest discrepancy or update appear first
    # Then alphabetical if tie:
    def sort_key(item):
        # item is (council_name, data_dict)
        return (
            item[1]["needs_update"],         # sort by needs_update (False < True)
            item[1]["discrepancies_count"],  # then by discrepancies
            item[0],                         # then by name
        )

    # We'll sort descending for "needs_update", so invert the boolean or reverse later
    sorted_councils = sorted(
        all_councils_data.items(),
        key=lambda x: (not x[1]["needs_update"], x[1]["discrepancies_count"], x[0])
    )

    for council, presence in sorted_councils:
        row = [
            council,
            "✔" if presence["in_files"] else "✘",
            "✔" if presence["in_json"] else "✘",
            "Yes" if presence["needs_update"] else "No",
            presence["discrepancies_count"],
        ]
        table_data.append(row)

    print(tabulate(table_data, headers=headers, tablefmt="grid"))

    # 5) Determine exit code:
    #    If any discrepancies OR any council needs updates -> fail
    if discrepancies_found or updates_needed:
        print("Some discrepancies found or updates are needed. Failing workflow.")
        sys.exit(1)
    else:
        print("No discrepancies found and no updates needed. Workflow successful.")


if __name__ == "__main__":
    # Optional CLI args: python script.py <repo> <branch>
    repo_arg = sys.argv[1] if len(sys.argv) > 1 else "robbrad/UKBinCollectionData"
    branch_arg = sys.argv[2] if len(sys.argv) > 2 else "master"
    main(repo_arg, branch_arg)



import re
import urllib.parse

import requests
from bs4 import BeautifulSoup, Tag

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = "https://fylde.gov.uk/resident/bins-recycling-and-rubbish/bin-collection-day"

        # Make the GET request
        session = requests.Session()
        response = session.get(URI)
        response.raise_for_status()

        soup: BeautifulSoup = BeautifulSoup(response.text, "html.parser")
        iframe = soup.find("iframe", id="bartec-iframe")
        if not iframe or not isinstance(iframe, Tag):
            raise Exception("Unexpected response from fylde.gov.uk")

        search_res = re.search(r"(?<=Token=)(.*?)(?=&|$)", str(iframe["src"]))
        if search_res is None:
            raise Exception("Token could not be extracted from fylde.gov.uk")
        token = search_res.group(1)

        if not token:
            raise Exception("Token could not be extracted from fylde.gov.uk")

        token = urllib.parse.unquote(token)

        parameters = {
            "Method": "calendareventsfromtoken",
            "Token": token,
            "UPRN": user_uprn,
        }

        API_URL = "https://collectiveview.bartec-systems.com/R152"
        API_METHOD = "GetData.ashx"

        response = session.get(f"{API_URL}/{API_METHOD}", params=parameters)
        response.raise_for_status()

        # Parse the JSON response
        bin_collection = response.json()
        if not (len(bin_collection) > 0 and "title" in bin_collection[0]):
            raise Exception("Unexpected response from fylde.gov.uk API")

        today = datetime.now()

        REGEX_JOB_NAME = r"(?i)Empty Bin\s+(?P<bin_type>\S+(?:\s+\S+)?)"
        # Loop through each collection in bin_collection
        for collection in bin_collection:
            bin_type = (
                re.search(REGEX_JOB_NAME, collection["title"])
                .group("bin_type")
                .strip()
                .title()
            )
            collection_date = datetime.fromtimestamp(
                int(collection["start"][6:-2]) / 1000
            )

            if collection_date < today:
                continue

            dict_data = {
                "type": bin_type,
                "collectionDate": collection_date.strftime("%d/%m/%Y"),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            source_date_format = "%d/%m/%Y"
            timeout = 10
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(
                "https://www.wokingham.gov.uk/rubbish-and-recycling/waste-collection/find-your-bin-collection-day"
            )

            # Wait for the postcode field to appear then populate it
            inputElement_postcode = WebDriverWait(driver, timeout).until(
                EC.presence_of_element_located((By.ID, "edit-postcode-search-csv"))
            )
            inputElement_postcode.send_keys(user_postcode)

            # Simulates hitting the "Enter" key to submit
            inputElement_postcode.send_keys(Keys.RETURN)

            # Select the exact address from the drop down box
            WebDriverWait(driver, timeout).until(
                EC.element_to_be_clickable(
                    (
                        By.XPATH,
                        ""
                        "//*[@id='edit-address-options-csv']//option[starts-with(normalize-space(.), '"
                        + user_paon
                        + "')]",
                    )
                )
            ).click()

            # Wait for the Show collection dates button to appear, then click it to get the collection dates
            inputElement_show_dates_button = WebDriverWait(driver, timeout).until(
                EC.presence_of_element_located(
                    (By.XPATH, '//*[@id="edit-show-collection-dates-csv"]')
                )
            )
            inputElement_show_dates_button.send_keys(Keys.RETURN)

            # Wait for the collection dates elements to load
            collection_date_cards = WebDriverWait(driver, timeout).until(
                EC.presence_of_all_elements_located(
                    (By.XPATH, '//div[@class = "card__content"]')
                )
            )

            for collection_date_card in collection_date_cards:
                waste_type = collection_date_card.find_element(
                    By.XPATH, './/h3[@class = "heading heading--sub heading--tiny"]'
                )
                collection_date = collection_date_card.find_element(
                    By.XPATH, './/span[@class = "card__date"]'
                )
                dt_collection_date = datetime.strptime(
                    collection_date.text.split(" ")[1], source_date_format
                )
                dict_data = {
                    "type": waste_type.text,
                    "collectionDate": dt_collection_date.strftime(date_format),
                }
                data["bins"].append(dict_data)
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import time

from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select, WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

import re
from datetime import datetime


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        try:
            # Make a BS4 object
            data = {"bins": []}

            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            headless = kwargs.get("headless")
            web_driver = kwargs.get("web_driver")
            driver = create_webdriver(web_driver, headless, None, __name__)
            page = "https://www.sholland.gov.uk/mycollections"
            check_uprn(user_uprn)
            check_postcode(user_postcode)
            driver.get(page)

            find_address_button = WebDriverWait(driver, timeout=15).until(
                EC.presence_of_element_located(
                    (By.ID, "SHDCWASTECOLLECTIONS_PAGE1_FINDADDRESSBUTTON")
                )
            )
            # Wait for the postcode field to appear then populate it
            postcode_input = WebDriverWait(driver, timeout=15).until(
                EC.presence_of_element_located(
                    (By.ID, "SHDCWASTECOLLECTIONS_PAGE1_POSTCODENEW")
                )
            )

            postcode_input.send_keys(user_postcode)

            find_address_button.click()

            # Wait until the address list contains an option with the target UPRN
            WebDriverWait(driver, timeout=20).until(
                lambda d: str(user_uprn)
                in [
                    option.get_attribute("value")
                    for option in Select(
                        d.find_element(By.ID, "SHDCWASTECOLLECTIONS_PAGE1_ADDRESSLIST")
                    ).options
                ]
            )

            # Now select the UPRN
            address_selection_menu = Select(
                driver.find_element(By.ID, "SHDCWASTECOLLECTIONS_PAGE1_ADDRESSLIST")
            )
            address_selection_menu.select_by_value(str(user_uprn))

            next_button = WebDriverWait(driver, timeout=15).until(
                EC.presence_of_element_located(
                    (By.ID, "SHDCWASTECOLLECTIONS_PAGE1_CONTINUEBUTTON_NEXT")
                )
            )

            next_button.click()

            # Check for text saying "Next collection dates"
            WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.XPATH, "//*[contains(text(), 'Check your collection days')]")
                )
            )

            soup = BeautifulSoup(driver.page_source, "html.parser")

            # Find the results container
            waste_blocks = soup.select("div.mycollections div.waste")

            for block in waste_blocks:
                try:
                    # Get the bin type
                    heading = block.find("h3").get_text(strip=True)
                    bin_type = heading.split(":")[0]  # e.g., "Refuse"

                    # Find the div with the next collection info
                    next_coll_div = block.find("div", class_="nextcoll")
                    if not next_coll_div:
                        continue

                    text = next_coll_div.get_text(" ", strip=True)

                    # Extract date using regex, e.g. 'Wednesday 9th April 2025'
                    match = re.search(
                        r"\b(\d{1,2})(?:st|nd|rd|th)?\s+([A-Za-z]+)\s+(\d{4})\b", text
                    )
                    if match:
                        day, month_str, year = match.groups()

                        # Convert to datetime to format as dd/mm/yyyy
                        date_obj = datetime.strptime(
                            f"{day} {month_str} {year}", "%d %B %Y"
                        )
                        formatted_date = date_obj.strftime("%d/%m/%Y")
                    else:
                        formatted_date = "Unknown"

                    data["bins"].append(
                        {"type": bin_type, "collectionDate": formatted_date}
                    )

                except Exception as parse_error:
                    print(f"Error parsing bin data: {parse_error}")

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception.
            if driver:
                driver.quit()
        return data



from datetime import datetime, timedelta

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = "https://el.hastings.gov.uk/MyArea/CollectionDays.asmx/LookupCollectionDaysByService"

        payload = {"Uprn": user_uprn}

        # Make the GET request
        response = requests.post(URI, json=payload, verify=False)
        response.raise_for_status()

        # Parse the JSON response
        bin_collection = response.json()["d"]

        # Loop through each collection in bin_collection
        for collection in bin_collection:
            bin_type = collection["Service"].removesuffix("collection service").strip()
            for collection_date in collection["Dates"]:
                collection_date = datetime.fromtimestamp(
                    int(collection_date.strip("/").removeprefix("Date").strip("()"))
                    / 1000
                ) + timedelta(hours=1)

                dict_data = {
                    "type": bin_type,
                    "collectionDate": collection_date.strftime("%d/%m/%Y"),
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        collections = []

        user_postcode = kwargs.get("postcode")
        user_uprn = kwargs.get("uprn")

        check_uprn(user_uprn)
        check_postcode(user_postcode)

        headers = {
            "authority": "www.rugby.gov.uk",
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "accept-language": "en-GB,en;q=0.9",
            "cache-control": "no-cache",
            "content-type": "application/x-www-form-urlencoded",
            # 'cookie': 'JSESSIONID=7E90CAB54B649C3DCC7F6B5DA0897C63; COOKIE_SUPPORT=true; GUEST_LANGUAGE_ID=en_GB; AWSELB=D941E98916B5759862ED6C39DA9FB3FD9880491851D200C98112ABEC3223D52B19A2A2C6B37A89D3650D44FA5728FCAFEDE7BB2592D948FFF9C7B18D76C41AF02C308B0F3A2DE17F1585E9959BCE68CC83BC3AC753; CookieControl={"necessaryCookies":[],"optionalCookies":{},"statement":{},"consentDate":1701710876715,"consentExpiry":90,"interactedWith":true,"user":"8FED8810-3C3E-4D50-A9DC-42655030B3B1"}',
            "origin": "https://www.rugby.gov.uk",
            "pragma": "no-cache",
            "referer": "https://www.rugby.gov.uk/check-your-next-bin-day",
            "sec-ch-ua": '"Chromium";v="118", "Opera GX";v="104", "Not=A?Brand";v="99"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
            "sec-fetch-dest": "document",
            "sec-fetch-mode": "navigate",
            "sec-fetch-site": "same-origin",
            "sec-fetch-user": "?1",
            "upgrade-insecure-requests": "1",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.5993.118 Safari/537.36",
        }
        params = {
            "p_p_id": "com_placecube_digitalplace_local_waste_portlet_CollectionDayFinderPortlet",
            "p_p_lifecycle": "0",
            "p_p_state": "normal",
            "p_p_mode": "view",
            "_com_placecube_digitalplace_local_waste_portlet_CollectionDayFinderPortlet_mvcRenderCommandName": "/collection_day_finder/get_days",
        }
        data = {
            "_com_placecube_digitalplace_local_waste_portlet_CollectionDayFinderPortlet_formDate": f"{datetime.now().timestamp().__floor__()}",
            "_com_placecube_digitalplace_local_waste_portlet_CollectionDayFinderPortlet_postcode": f"{user_postcode}",
            "_com_placecube_digitalplace_local_waste_portlet_CollectionDayFinderPortlet_uprn": f"{user_uprn}",
        }

        response = requests.post(
            "https://www.rugby.gov.uk/check-your-next-bin-day",
            params=params,
            headers=headers,
            data=data,
        )

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        table_rows = soup.find("table", {"class": "table"}).find("tbody").find_all("tr")

        for row in table_rows:
            row_text = row.text.strip().split("\n")
            bin_text = row_text[0].split(" ")
            bin_type = " ".join(bin_text[1:]).capitalize()
            collections.append(
                (bin_type, datetime.strptime(row_text[1], "%A %d %b %Y"))
            )
            collections.append(
                (bin_type, datetime.strptime(row_text[3], "%A %d %b %Y"))
            )

        ordered_data = sorted(collections, key=lambda x: x[1])
        data = {"bins": []}
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")
        headless = kwargs.get("headless")
        check_uprn(user_uprn)
        check_postcode(user_postcode)

        # Create Selenium webdriver
        driver = create_webdriver(web_driver, headless, None, __name__)
        driver.get("https://www.staffsmoorlands.gov.uk/findyourbinday")

        # Close cookies banner
        # cookieAccept = WebDriverWait(driver, 10).until(
        #    EC.presence_of_element_located(
        #        (By.CSS_SELECTOR, ".cookiemessage__link--close")
        #    )
        # )
        # cookieAccept.click()

        # Wait for the postcode field to appear then populate it
        inputElement_postcode = WebDriverWait(driver, 30).until(
            EC.presence_of_element_located(
                (By.ID, "FINDBINDAYSSTAFFORDSHIREMOORLANDS_POSTCODESELECT_POSTCODE")
            )
        )
        inputElement_postcode.send_keys(user_postcode)

        # Click search button
        findAddress = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (
                    By.ID,
                    "FINDBINDAYSSTAFFORDSHIREMOORLANDS_POSTCODESELECT_PAGE1NEXT_NEXT",
                )
            )
        )
        findAddress.click()

        # Wait for the 'Select address' dropdown to appear and select option matching UPRN
        dropdown = WebDriverWait(driver, 30).until(
            EC.presence_of_element_located(
                (By.ID, "FINDBINDAYSSTAFFORDSHIREMOORLANDS_ADDRESSSELECT_ADDRESS")
            )
        )
        # Create a 'Select' for it, then select the matching URPN option
        dropdownSelect = Select(dropdown)
        dropdownSelect.select_by_value(user_uprn)

        # Wait for the submit button to appear, then click it to get the collection dates
        submit = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (
                    By.ID,
                    "FINDBINDAYSSTAFFORDSHIREMOORLANDS_ADDRESSSELECT_ADDRESSSELECTNEXTBTN_NEXT",
                )
            )
        )
        submit.click()

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        # Quit Selenium webdriver to release session
        driver.quit()

        # Get months
        for month_wrapper in soup.find_all("div", {"class": "bin-collection__month"}):
            if month_wrapper:
                month_year = month_wrapper.find(
                    "h3", {"class": "bin-collection__title"}
                ).get_text(strip=True)
                # Get collections
                for collection in month_wrapper.find_all(
                    "li", {"class": "bin-collection__item"}
                ):
                    day = collection.find(
                        "span", {"class": "bin-collection__number"}
                    ).get_text(strip=True)
                    if month_year and day:
                        bin_date = datetime.strptime(day + " " + month_year, "%d %B %Y")
                        dict_data = {
                            "type": collection.find(
                                "span", {"class": "bin-collection__type"}
                            ).get_text(strip=True),
                            "collectionDate": bin_date.strftime(date_format),
                        }
                        data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return data



from datetime import datetime

import requests
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            # Get and check UPRN
            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            check_uprn(user_uprn)
            check_postcode(user_postcode)
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            bindata = {"bins": []}

            API_URL = "https://secure.ashford.gov.uk/waste/collectiondaylookup/"

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(API_URL)

            # Wait for the postcode field to appear then populate it
            inputElement_postcode = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.ID, "ContentPlaceHolder1_CollectionDayLookup2_TextBox_PostCode")
                )
            )
            inputElement_postcode.send_keys(user_postcode)

            # Click search button
            findAddress = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.ID,
                        "ContentPlaceHolder1_CollectionDayLookup2_Button_PostCodeSearch",
                    )
                )
            )
            findAddress.click()

            # Wait for the 'Select your property' dropdown to appear and select the first result
            dropdown = WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (
                        By.ID,
                        "ContentPlaceHolder1_CollectionDayLookup2_DropDownList_Addresses",
                    )
                )
            )

            # Create a 'Select' for it, then select the first address in the list
            # (Index 0 is "Make a selection from the list")
            dropdownSelect = Select(dropdown)
            dropdownSelect.select_by_value(str(user_uprn))

            # Click search button
            findAddress = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.ID,
                        "ContentPlaceHolder1_CollectionDayLookup2_Button_SelectAddress",
                    )
                )
            )
            findAddress.click()

            h4_element = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.XPATH, "//h4[contains(text(), 'Collection Dates')]")
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            bin_tables = soup.find_all("table")

            for bin_table in bin_tables:
                bin_text = bin_table.find(
                    "td", id=re.compile("CollectionDayLookup2_td_")
                )
                if not bin_text:
                    continue

                bin_type_soup = bin_text.find("b")

                if not bin_type_soup:
                    continue
                bin_type: str = bin_type_soup.text.strip().split(" (")[0]

                date_soup = bin_text.find(
                    "span", id=re.compile(r"CollectionDayLookup2_Label_\w*_Date")
                )
                if not date_soup or (
                    " " not in date_soup.text.strip()
                    and date_soup.text.strip().lower() != "today"
                ):
                    continue
                date_str: str = date_soup.text.strip()
                try:
                    if date_soup.text.strip().lower() == "today":
                        date = datetime.now().date()
                    else:
                        date = datetime.strptime(
                            date_str.split(" ")[1], "%d/%m/%Y"
                        ).date()

                except ValueError:
                    continue

                dict_data = {
                    "type": bin_type,
                    "collectionDate": date.strftime("%d/%m/%Y"),
                }
                bindata["bins"].append(dict_data)

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return bindata



import re
import time

import requests
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)
            bindata = {"bins": []}

            URI_1 = "https://www.hertsmere.gov.uk/Environment-Refuse-and-Recycling/Recycling--Waste/Bin-collections/Collections-and-calendar.aspx"
            URI_2 = "https://hertsmere-services.onmats.com/w/webpage/round-search"

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(URI_1)

            soup = BeautifulSoup(driver.page_source, "html.parser")

            current_week = (soup.find("li", class_="current")).text.strip()

            strong = soup.find_all("strong", text=re.compile(r"^Week"))

            bin_weeks = []
            for tag in strong:
                parent = tag.parent
                bin_type = (
                    (parent.text)
                    .split("-")[1]
                    .strip()
                    .replace("\xa0", " ")
                    .split(" and ")
                )
                for bin in bin_type:
                    dict_data = {
                        "week": tag.text.replace("\xa0", " "),
                        "bin_type": bin,
                    }
                    bin_weeks.append(dict_data)

            driver.get(URI_2)

            # Wait for the postcode field to appear then populate it
            inputElement_postcode = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (
                        By.CLASS_NAME,
                        "relation_path_type_ahead_search",
                    )
                )
            )
            inputElement_postcode.send_keys(user_postcode)

            WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (
                        By.XPATH,
                        f"//ul[@class='result_list']/li[starts-with(@aria-label, '{user_paon}')]",
                    )
                )
            ).click()

            WebDriverWait(driver, timeout=10).until(
                EC.element_to_be_clickable(
                    (
                        By.CSS_SELECTOR,
                        "input.fragment_presenter_template_edit.btn.bg-primary.btn-medium[type='submit']",
                    )
                )
            ).click()

            WebDriverWait(driver, timeout=10).until(
                EC.presence_of_element_located(
                    (By.XPATH, "//h3[contains(text(), 'Collection days')]")
                )
            )

            soup = BeautifulSoup(driver.page_source, "html.parser")

            table = soup.find("table", class_="table listing table-striped")

            # Check if the table was found
            if table:
                # Extract table rows and cells
                table_data = []
                for row in table.find("tbody").find_all("tr"):
                    # Extract cell data from each <td> tag
                    row_data = [
                        cell.get_text(strip=True) for cell in row.find_all("td")
                    ]
                    table_data.append(row_data)

            else:
                print("Table not found.")

            collection_day = (table_data[0])[1]

            current_week_bins = [
                bin for bin in bin_weeks if bin["week"] == current_week
            ]
            next_week_bins = [bin for bin in bin_weeks if bin["week"] != current_week]

            days_of_week = [
                "Monday",
                "Tuesday",
                "Wednesday",
                "Thursday",
                "Friday",
                "Saturday",
                "Sunday",
            ]

            today = datetime.now()
            today_idx = today.weekday()  # Monday is 0 and Sunday is 6
            target_idx = days_of_week.index(collection_day)

            days_until_target = (target_idx - today_idx) % 7
            if days_until_target == 0:
                next_day = today
            else:
                next_day = today + timedelta(days=days_until_target)

            current_week_dates = get_dates_every_x_days(next_day, 14, 7)
            next_week_date = next_day + timedelta(days=7)
            next_week_dates = get_dates_every_x_days(next_week_date, 14, 7)

            for date in current_week_dates:
                for bin in current_week_bins:
                    dict_data = {
                        "type": bin["bin_type"],
                        "collectionDate": date,
                    }
                    bindata["bins"].append(dict_data)

            for date in next_week_dates:
                for bin in next_week_bins:
                    dict_data = {
                        "type": bin["bin_type"],
                        "collectionDate": date,
                    }
                    bindata["bins"].append(dict_data)

            bindata["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
            )

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return bindata



from datetime import datetime

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get and check UPRN
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        COLLECTION_MAP = {
            "ahtm_dates_black_bin": "Black bin",
            "ahtm_dates_brown_commingled_bin": "Brown bin",
            "ahtm_dates_blue_pulpable_bin": "Blue bin",
            "ahtm_dates_green_organic_bin": "Green Bin",
        }

        API_URL = "https://manchester.form.uk.empro.verintcloudservices.com/api/custom?action=bin_checker-get_bin_col_info&actionedby=_KDF_custom&loadform=true&access=citizen&locale=en"
        AUTH_URL = "https://manchester.form.uk.empro.verintcloudservices.com/api/citizen?archived=Y&preview=false&locale=en"
        AUTH_KEY = "Authorization"

        r = requests.get(AUTH_URL)
        r.raise_for_status()
        auth_token = r.headers[AUTH_KEY]

        post_data = {
            "name": "sr_bin_coll_day_checker",
            "data": {
                "uprn": user_uprn,
                "nextCollectionFromDate": (datetime.now() - timedelta(days=1)).strftime(
                    "%Y-%m-%d"
                ),
                "nextCollectionToDate": (datetime.now() + timedelta(days=30)).strftime(
                    "%Y-%m-%d"
                ),
            },
            "email": "",
            "caseid": "",
            "xref": "",
            "xref1": "",
            "xref2": "",
        }

        headers = {
            "referer": "https://manchester.portal.uk.empro.verintcloudservices.com/",
            "accept": "application/json",
            "content-type": "application/json",
            AUTH_KEY: auth_token,
        }

        r = requests.post(API_URL, data=json.dumps(post_data), headers=headers)
        r.raise_for_status()

        result = r.json()
        print(result["data"])

        for key, value in result["data"].items():
            if key.startswith("ahtm_dates_"):
                print(key)
                print(value)

                dates_list = [
                    datetime.strptime(date.strip(), "%d/%m/%Y %H:%M:%S").date()
                    for date in value.split(";")
                    if date.strip()
                ]

                for current_date in dates_list:
                    dict_data = {
                        "type": COLLECTION_MAP.get(key),
                        "collectionDate": current_date.strftime(date_format),
                    }
                    bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )
        return bindata



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        user_uprn = str(user_uprn).zfill(12)

        URI = "https://biffaleicester.co.uk/wp-admin/admin-ajax.php"

        payload = {
            "action": "get_details_api",
            "uprn": user_uprn,
        }

        headers = {
            "Origin": "https://biffaleicester.co.uk",
            "Referer": "https://biffaleicester.co.uk/services/waste-collection-days/",
            "User-Agent": "Mozilla/5.0",
        }

        # Make the GET request
        response = requests.post(URI, headers=headers, data=payload)

        # Parse the JSON response
        bin_collection = response.json()

        # Loop through each collection in bin_collection
        for collection in bin_collection["anyType"]:
            bin_type = collection["ServiceModeDesc"]
            date = collection["ServiceDueDate"]

            dict_data = {
                "type": bin_type,
                "collectionDate": datetime.strptime(
                    date,
                    "%d/%m/%y",
                ).strftime(date_format),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return bindata



from xml.etree import ElementTree

from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    baseclass. They can also override some
    operations with a default implementation.
    """

    def parse_data(self, page, **kwargs) -> dict:
        """This method makes the request to the council

        Keyword arguments:
        url -- the url to get the data from
        """
        # Set a user agent so we look like a browser ;-)
        user_agent = "Mozilla/5.0 (Windows NT 6.1; Win64; x64)"
        headers = {"User-Agent": user_agent, "Content-Type": "text/xml"}

        uprn = kwargs.get("uprn")
        try:
            if uprn is None or uprn == "":
                raise ValueError("Invalid UPRN")
        except Exception as ex:
            print(f"Exception encountered: {ex}")
            print(
                "Please check the provided UPRN. If this error continues, please first trying setting the "
                "UPRN manually on line 115 before raising an issue."
            )

        # Make the Request - change the URL - find out your property number
        # URL
        url = "https://collections-torridge.azurewebsites.net/WebService2.asmx"
        # Post data
        post_data = (
            '<?xml version="1.0" encoding="utf-8"?><soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"><soap:Body><getRoundCalendarForUPRN xmlns="http://tempuri2.org/"><council>TOR</council><UPRN>'
            + uprn
            + "</UPRN><PW>wax01653</PW></getRoundCalendarForUPRN></soap:Body></soap:Envelope>"
        )
        requests.packages.urllib3.disable_warnings()
        page = requests.post(url, headers=headers, data=post_data)

        # Remove the soap wrapper
        namespaces = {
            "soap": "http://schemas.xmlsoap.org/soap/envelope/",
            "a": "http://tempuri2.org/",
        }
        dom = ElementTree.fromstring(page.text)
        page = dom.find(
            "./soap:Body"
            "/a:getRoundCalendarForUPRNResponse"
            "/a:getRoundCalendarForUPRNResult",
            namespaces,
        )
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        b_el = soup.find("b", string="GardenBin")
        if b_el:
            results = re.search(
                "([A-Za-z]+ \\d\\d? [A-Za-z]+) (.*?)", b_el.next_sibling.split(": ")[1]
            )
            if results and results.groups()[0]:
                date = results.groups()[0] + " " + datetime.today().strftime("%Y")
                data["bins"].append(
                    {
                        "type": "GardenBin",
                        "collectionDate": get_next_occurrence_from_day_month(
                            datetime.strptime(date, "%a %d %b %Y")
                        ).strftime(date_format),
                    }
                )

        b_el = soup.find("b", string="Refuse")
        if b_el:
            results = re.search(
                "([A-Za-z]+ \\d\\d? [A-Za-z]+) (.*?)", b_el.next_sibling.split(": ")[1]
            )
            if results and results.groups()[0]:
                date = results.groups()[0] + " " + datetime.today().strftime("%Y")
                data["bins"].append(
                    {
                        "type": "Refuse",
                        "collectionDate": get_next_occurrence_from_day_month(
                            datetime.strptime(date, "%a %d %b %Y")
                        ).strftime(date_format),
                    }
                )

        b_el = soup.find("b", string="Recycling")
        if b_el:
            results = re.search(
                "([A-Za-z]+ \\d\\d? [A-Za-z]+) (.*?)", b_el.next_sibling.split(": ")[1]
            )
            if results and results.groups()[0]:
                date = results.groups()[0] + " " + datetime.today().strftime("%Y")
                data["bins"].append(
                    {
                        "type": "Recycling",
                        "collectionDate": get_next_occurrence_from_day_month(
                            datetime.strptime(date, "%a %d %b %Y")
                        ).strftime(date_format),
                    }
                )

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        user_uprn = str(user_uprn).zfill(12)
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://www.hinckley-bosworth.gov.uk/set-location?id={user_uprn}&redirect=refuse&rememberloc="

        # Make the GET request
        response = requests.get(URI)

        # Parse the HTML
        soup = BeautifulSoup(response.content, "html.parser")

        # Find all the bin collection date containers
        bin_schedule = []
        collection_divs = soup.find_all(
            "div", class_=["first_date_bins", "last_date_bins"]
        )

        for div in collection_divs:
            # Extract the date
            date = div.find("h3", class_="collectiondate").text.strip().replace(":", "")

            # Extract bin types
            bins = [img["alt"] for img in div.find_all("img", class_="collection")]

            # Append to the schedule
            bin_schedule.append({"date": date, "bins": bins})

        current_year = datetime.now().year
        current_month = datetime.now().month

        # Print the schedule
        for entry in bin_schedule:
            bin_types = entry["bins"]
            date = datetime.strptime(entry["date"], "%d %B")

            if (current_month > 9) and (date.month < 4):
                date = date.replace(year=(current_year + 1))
            else:
                date = date.replace(year=current_year)

            for bin_type in bin_types:

                dict_data = {
                    "type": bin_type,
                    "collectionDate": date.strftime("%d/%m/%Y"),
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



from datetime import datetime

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")

        check_uprn(user_uprn)
        check_postcode(user_postcode)

        # Start a new session to walk through the form
        requests.packages.urllib3.disable_warnings()
        s = requests.Session()

        headers = {
            "Origin": "https://www.centralbedfordshire.gov.uk",
            "Referer": "https://www.centralbedfordshire.gov.uk/info/163/bins_and_waste_collections_-_check_bin_collection_day",
        }

        files = {
            "postcode": (None, user_postcode),
            "address": (None, user_uprn),
        }

        response = requests.post(
            "https://www.centralbedfordshire.gov.uk/info/163/bins_and_waste_collections_-_check_bin_collection_day#my_bin_collections",
            headers=headers,
            files=files,
        )

        # Make that BS4 object and use it to prettify the response
        soup = BeautifulSoup(response.content, features="html.parser")
        soup.prettify()

        collections_div = soup.find(id="collections")

        # Get the collection items on the page and strip the bits of text that we don't care for
        collections = []
        for bin in collections_div.find_all("h3"):
            next_bin = bin.next_sibling

            while next_bin.name != "h3" and next_bin.name != "p":
                if next_bin.name != "br":
                    collection_date = datetime.strptime(bin.text, "%A, %d %B %Y")
                    collections.append((next_bin, collection_date))
                next_bin = next_bin.next_sibling

        # Sort the collections by date order rather than bin type, then return as a dictionary (with str date)
        ordered_data = sorted(collections, key=lambda x: x[1])
        data = {"bins": []}
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://my.blaby.gov.uk/set-location.php?ref={user_uprn}&redirect=collections"

        # Make the GET request
        response = requests.get(URI)

        # Parse the HTML
        soup = BeautifulSoup(response.content, "html.parser")

        # Find each collection container based on the class "box-item"
        for container in soup.find_all(class_="box-item"):

            # Get the next collection dates from the <p> tag containing <strong>
            try:
                dates_tag = (
                    container.find("p", string=lambda text: "Next" in text)
                    .find_next("p")
                    .find("strong")
                )
            except:
                continue
            collection_dates = (
                dates_tag.text.strip().split(", and then ")
                if dates_tag
                else "No dates found"
            )

            for collection_date in collection_dates:
                dict_data = {
                    "type": container.find("h2").text.strip(),
                    "collectionDate": collection_date,
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        data = {"bins": []}

        baseurl = "https://services.southwark.gov.uk/bins/lookup/"
        url = baseurl + user_uprn

        headers = {
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36"
        }

        # Make the web request
        response = requests.get(url, headers=headers).text

        soup = BeautifulSoup(response, "html.parser")

        # Extract recycling collection information
        recycling_section = soup.find(
            "div", {"aria-labelledby": "recyclingCollectionTitle"}
        )
        if recycling_section:
            recycling_title = recycling_section.find(
                "p", {"id": "recyclingCollectionTitle"}
            ).text
            recycling_next_collection = (
                recycling_section.find(text=lambda text: "Next collection" in text)
                .strip()
                .split(": ")[1]
            )

            dict_data = {
                "type": recycling_title,
                "collectionDate": datetime.strptime(
                    recycling_next_collection, "%a, %d %B %Y"
                ).strftime("%d/%m/%Y"),
            }
            data["bins"].append(dict_data)

        # Extract refuse collection information
        refuse_section = soup.find("div", {"aria-labelledby": "refuseCollectionTitle"})
        if refuse_section:
            refuse_title = refuse_section.find(
                "p", {"id": "refuseCollectionTitle"}
            ).text
            refuse_next_collection = (
                refuse_section.find(text=lambda text: "Next collection" in text)
                .strip()
                .split(": ")[1]
            )

            dict_data = {
                "type": refuse_title,
                "collectionDate": datetime.strptime(
                    refuse_next_collection, "%a, %d %B %Y"
                ).strftime("%d/%m/%Y"),
            }
            data["bins"].append(dict_data)

        # Extract food waste collection information
        food_section = soup.find(
            "div", {"aria-labelledby": "domesticFoodCollectionTitle"}
        )
        if food_section:
            food_title = food_section.find(
                "p", {"id": "domesticFoodCollectionTitle"}
            ).text
            food_next_collection = (
                food_section.find(text=lambda text: "Next collection" in text)
                .strip()
                .split(": ")[1]
            )

            dict_data = {
                "type": food_title,
                "collectionDate": datetime.strptime(
                    food_next_collection, "%a, %d %B %Y"
                ).strftime("%d/%m/%Y"),
            }
            data["bins"].append(dict_data)

        comrec_section = soup.find(
            "div", {"aria-labelledby": "recyclingCommunalCollectionTitle"}
        )
        if comrec_section:
            comrec_title = comrec_section.find(
                "p", {"id": "recyclingCommunalCollectionTitle"}
            ).text
            comrec_next_collection = (
                comrec_section.find(text=lambda text: "Next collection" in text)
                .strip()
                .split(": ")[1]
            )

            dict_data = {
                "type": comrec_title,
                "collectionDate": datetime.strptime(
                    comrec_next_collection, "%a, %d %B %Y"
                ).strftime("%d/%m/%Y"),
            }
            data["bins"].append(dict_data)

        comref_section = soup.find(
            "div", {"aria-labelledby": "refuseCommunalCollectionTitle"}
        )
        if comref_section:
            comref_title = comref_section.find(
                "p", {"id": "refuseCommunalCollectionTitle"}
            ).text
            comref_next_collection = (
                comref_section.find(text=lambda text: "Next collection" in text)
                .strip()
                .split(": ")[1]
            )

            dict_data = {
                "type": comref_title,
                "collectionDate": datetime.strptime(
                    comref_next_collection, "%a, %d %B %Y"
                ).strftime("%d/%m/%Y"),
            }
            data["bins"].append(dict_data)

        return data



import time

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        user_uprn = user_uprn.zfill(12)
        bindata = {"bins": []}

        URI = "https://www.argyll-bute.gov.uk/rubbish-and-recycling/household-waste/bin-collection"

        data = {"addressSelect": user_uprn}

        s = requests.session()
        r = s.post(URI, data=data)
        r.raise_for_status()

        soup = BeautifulSoup(r.content, features="html.parser")
        soup.prettify()

        # Find the table and extract the rows with bin schedule information
        table = soup.find("table", class_="table table-bordered")
        rows = table.find_all("tr")[1:]  # Skip the header row

        current_year = datetime.now().year
        # Loop through each row and extract the bin type and collection date
        for row in rows:
            cells = row.find_all("td")
            bin_type = cells[0].get_text(strip=True)
            collection_date = cells[1].get_text(strip=True)

            collection_date = datetime.strptime(
                collection_date,
                "%A %d %B",
            )

            if collection_date.month == 1:
                collection_date = collection_date.replace(year=current_year + 1)
            else:
                collection_date = collection_date.replace(year=current_year)

            dict_data = {
                "type": bin_type,
                "collectionDate": collection_date.strftime(date_format),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return bindata



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        # Function to extract bin collection information
        def extract_bin_schedule(soup, heading_class):
            collections = []

            # Find the relevant section based on the heading class
            section_heading = soup.find("div", class_=heading_class)
            if section_heading:
                # Find all the bin collection dates in that section
                collection_dates = section_heading.find_next(
                    "div", class_="col-sm-12 col-md-9"
                ).find_all("h4")
                for date in collection_dates:
                    # Clean and add the date to the list
                    collections.append(date.get_text(strip=True))

            return collections

        # URL for bin collection schedule
        url = f"https://www.armaghbanbridgecraigavon.gov.uk/resident/binday-result/?address={user_uprn}"

        # Send a GET request to fetch the page content
        response = requests.get(url)

        # Check if the request was successful
        if response.status_code == 200:
            # Parse the page content using BeautifulSoup
            soup = BeautifulSoup(response.text, "html.parser")

            # Extract bin collection schedules by their sections
            domestic_collections = extract_bin_schedule(soup, "heading bg-black")
            for collection in domestic_collections:
                bindata["bins"].append(
                    {"collectionDate": collection, "type": "Domestic"}
                )
            recycling_collections = extract_bin_schedule(soup, "heading bg-green")
            for collection in recycling_collections:
                bindata["bins"].append(
                    {"collectionDate": collection, "type": "Recycling"}
                )
            garden_collections = extract_bin_schedule(soup, "heading bg-brown")
            for collection in garden_collections:
                bindata["bins"].append({"collectionDate": collection, "type": "Garden"})

        else:
            print(f"Failed to retrieve data. Status code: {response.status_code}")

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from datetime import datetime

from bs4 import BeautifulSoup
from dateutil.relativedelta import relativedelta
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import create_webdriver
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            bin_data_dict = {"bins": []}
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            data = {"bins": []}

            # Get our initial session running
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(kwargs.get("url"))

            wait = WebDriverWait(driver, 30)
            wait.until(
                EC.presence_of_element_located((By.CLASS_NAME, "waste-service-image"))
            )

            # Parse the HTML content
            soup = BeautifulSoup(driver.page_source, "html.parser")

            # Find all elements with class 'govuk-summary-list'
            waste_services = soup.find_all(
                "h3", class_="govuk-heading-m waste-service-name"
            )

            for service in waste_services:
                service_title = service.get_text(strip=True)
                next_collection = service.find_next_sibling().find(
                    "dt", string="Next collection"
                )

                if next_collection:
                    next_collection_date = next_collection.find_next_sibling().get_text(
                        strip=True
                    )
                    # Extract date part and remove the suffix
                    next_collection_date_parse = next_collection_date.split(",")[
                        1
                    ].strip()
                    day, month = next_collection_date_parse.split()[:2]

                    # Remove the suffix (e.g., 'th', 'nd', 'rd', 'st') from the day
                    if day.endswith(("th", "nd", "rd", "st")):
                        day = day[:-2]  # Remove the last two characters

                    # Reconstruct the date string without the suffix
                    date_without_suffix = f"{day} {month}"

                    # Parse the date string to a datetime object
                    date_object = datetime.strptime(date_without_suffix, "%d %B")

                    # Get the current year
                    current_year = datetime.now().year

                    # Append the year to the date
                    date_with_year = date_object.replace(year=current_year)

                    # Check if the parsed date is in the past compared to the current date
                    if date_object < datetime.now():
                        # If the parsed date is in the past, assume it's for the next year
                        current_year += 1

                    # Format the date with the year
                    date_with_year_formatted = date_with_year.strftime(
                        "%d/%m/%Y"
                    )  # Format the date as '%d/%m/%Y'

                    # Create the dictionary with the formatted data
                    dict_data = {
                        "type": service_title,
                        "collectionDate": date_with_year_formatted,
                    }
                    data["bins"].append(dict_data)
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



# This script pulls (in one hit) the data from Merton Council Bins Data
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# Council class for Merton Council
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}
        collections = []

        # Search for the specific bin in the table using BS4
        rows = soup.find("table", class_=("collectiondays")).find_all(
            "tr",
            class_=(
                "food-caddy",
                "papercard-wheelie",
                "plastics-boxes",
                "rubbish-wheelie",
                "textiles",
                "batteries",
                "garden",
                "communal-recycling",
                "rubbish-communal",
            ),
        )

        possible_formats = [
            "%d %B %Y",
            "%A %d %B %Y",
        ]

        # Loops the Rows
        for row in rows:
            # Get all the cells
            cells = row.find_all("td")
            # First cell is the bin_type
            bin_type = cells[0].get_text().strip()

            # Garden waste is optional, so skip if none scheduled - causes an error if it gets into the date parsing below.
            if bin_type == "Garden waste":
                if (
                    "There are no garden waste collections scheduled for this address."
                    in cells[1].select("p")[0].get_text().strip()
                ):
                    continue

            # Date is on the second cell, second paragraph, wrapped in p
            collectionDate = None
            for format in possible_formats:
                try:
                    collectionDate = datetime.strptime(
                        cells[1].select("p > b")[2].get_text(strip=True), format
                    )
                    break  # Exit the loop if parsing is successful
                except ValueError:
                    continue

            # Add each collection to the list as a tuple
            collections.append((bin_type, collectionDate))

        ordered_data = sorted(collections, key=lambda x: x[1])
        for item in ordered_data:
            dict_data = {
                "type": item[0].capitalize(),
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



from datetime import datetime, timedelta

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://maps.epsom-ewell.gov.uk/myeebc.aspx?action=SetAddress&UniqueId={user_uprn}"

        # Make the GET request
        response = requests.get(URI)

        soup = BeautifulSoup(response.text, "html.parser")

        # print(soup)

        div = soup.find_all("div", class_="atPanelContent atAlt1 atLast")

        # print(div[1])

        panels = div[1].find_all("div", class_="atPanelData")

        # print(panels)

        def get_full_date(date_str):
            # Get the current year
            current_year = datetime.today().year

            # Convert the input string to a datetime object (assuming the current year first)
            date_obj = datetime.strptime(f"{date_str} {current_year}", "%A %d %B %Y")

            # If the date has already passed this year, use next year
            if date_obj < datetime.today():
                date_obj = datetime.strptime(
                    f"{date_str} {current_year + 1}", "%A %d %B %Y"
                )

            return date_obj.strftime(date_format)  # Return in YYYY-MM-DD format

        for panel in panels:
            bin_type_tag = panel.find("h4")  # Extracts bin type
            date_text = panel.find_all("td")  # Extracts collection date

            date_text = date_text[1]

            if bin_type_tag and date_text:
                bin_type = bin_type_tag.text.strip()
                try:
                    collection_date = date_text.text.strip().split(":")[1]
                except IndexError:
                    continue

                bin_type = (
                    (" ".join(bin_type.splitlines())).replace("  ", " ")
                ).lstrip()
                collection_date = (
                    (" ".join(collection_date.splitlines())).replace("  ", " ")
                ).lstrip()

                dict_data = {
                    "type": bin_type,
                    "collectionDate": get_full_date(collection_date),
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata
